/**
 *  @file: GetFuncAddr.cpp
 *
 *  @project get function addresses for WinCE/PocketPC
 *
 *  @author Collin Mulliner <collin-ppc(at)betaversion.net>
 *
 *  This is Free Software under the GPL!
 */

#include "stdafx.h"
#include <winbase.h>
#include <stdio.h>

int WINAPI WinMain(	HINSTANCE hInstance,
					HINSTANCE hPrevInstance,
					LPTSTR    lpCmdLine,
					int       nCmdShow)
{
	char input[] = "/GetFuncAddr.in";
	char output[] = "/GetFuncAddr.out";
	OSVERSIONINFO osvi;
	SYSTEM_INFO sysi;
	char version_str[1024];
	char system_str[1024];
	char info_str[2048];


	GetSystemInfo(&sysi);
	if (sysi.wProcessorArchitecture == PROCESSOR_ARCHITECTURE_ARM) {
		sprintf(system_str, "ARM Revision %d", sysi.wProcessorRevision);
	}

	osvi.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
	if (GetVersionEx(&osvi) == 0) {
		MessageBox(NULL, TEXT("Failed to get OS information!"), TEXT("Error"), 0);
	}
	else {
		if (osvi.dwPlatformId != VER_PLATFORM_WIN32_CE) {
			MessageBox(NULL, TEXT("Non WinCE device!"), TEXT("Error"), 0);
		}
		
		sprintf(version_str, "WinCE %d.%d Build %d", osvi.dwMajorVersion, osvi.dwMinorVersion, osvi.dwBuildNumber);
	}

	// display device info
	TCHAR display[2048];
	strcpy(info_str, version_str);
	strcat(info_str, " on ");
	strcat(info_str, system_str);
	strcat(info_str, "\n\nOutput will be written to: /GetFuncAddr.out\n Please manually add the device type and manufacturer, thanks!");
	mbstowcs(display, info_str, 2048);
	MessageBox(NULL, display, TEXT("Device Info"), MB_ICONINFORMATION|MB_OK);

	FILE *in = fopen(input, "r");
	if (in == NULL) {
		MessageBox(NULL, TEXT("Can't open input at: /GetFuncAddr.in"), TEXT("Error"), 0);
		return(0);
	}
	FILE *out = fopen(output, "w+");
	if (out == NULL) {
		MessageBox(NULL, TEXT("Can't open output at: /GetFuncAddr.out"), TEXT("Error"), 0);
		return(0);
	}

	fprintf(out, "==Generated by GetFuncAddr v0.0.1 by Collin Mulliner <collin-ppc(AT)betaversion.net>==\n\n");
	fprintf(out, "System Info: %s on %s\n\n", version_str, system_str);

	while (!feof(in)) {
		char line[1024];
		char lib_str[512];
		char func_str[512];
		TCHAR lib_str_w[512];
		TCHAR func_str_w[512];
		HINSTANCE lib_h;
		FARPROC p_addr;

		if (fgets(line, 1024, in) == NULL) break;
		switch (line[0]) {
		case '#':
		case '\r':
		case '\n':
		case ' ':
			if (line[1] == '#') fprintf(out, "%s", line);
			continue;
		}
		sscanf(line, "%s %s", lib_str, func_str);

		mbstowcs(lib_str_w, lib_str, 512);
		mbstowcs(func_str_w, func_str, 512);

		lib_h = LoadLibrary(lib_str_w);

		if (lib_h == NULL) {
			fprintf(out, "%s:%s (Error) Library not installed!\n", lib_str, func_str);
		}
		else {
			p_addr = GetProcAddress(lib_h, func_str_w);
			if (p_addr == NULL) {
				fprintf(out, "%s:%s (Error) Function not found!\n", lib_str, func_str);
			}
			else {
				unsigned char a[4];
				memcpy(a, (unsigned char*)&p_addr, 4);
				fprintf(out, "%s:%s 0x%0.4x (0x%0.2x,0x%0.2x,0x%0.2x,0x%0.2x,)\n", lib_str, func_str, p_addr, a[0],a[1],a[2],a[3]);
			}
		}
	}

	fclose(in);
	fclose(out);

	return(0);
}

